=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-03-04 23:04:37 +0000
+++ bzrlib/commit.py	2006-03-06 22:11:12 +0000
@@ -232,8 +232,8 @@
         self.rev_id = rev_id
         self.specific_files = specific_files
         self.allow_pointless = allow_pointless
-        self.revprops = {'branch-nick': self.branch.nick}
-        if revprops:
+        self.revprops = {}
+        if revprops is not None:
             self.revprops.update(revprops)
 
         self.work_tree.lock_write()

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-03-06 07:11:09 +0000
+++ bzrlib/errors.py	2006-03-07 05:59:14 +0000
@@ -745,6 +745,10 @@
         self.tree = tree
 
 
+class MergeModifiedFormatError(BzrNewError):
+    """Error in merge modified format"""
+
+
 class CorruptRepository(BzrNewError):
     """An error has been detected in the repository %(repo_path)s.
 Please run bzr reconcile on this repository."""

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-03-06 17:47:48 +0000
+++ bzrlib/merge.py	2006-03-07 19:46:39 +0000
@@ -350,13 +350,26 @@
             self.cook_conflicts(fs_conflicts)
             for line in conflicts_strings(self.cooked_conflicts):
                 warning(line)
-            self.tt.apply()
+            results = self.tt.apply()
         finally:
             try:
                 self.tt.finalize()
             except:
                 pass
-       
+        self.write_modified(results)
+
+    def write_modified(self, results):
+        modified_hashes = {}
+        for path in results.modified_paths:
+            file_id = self.this_tree.path2id(self.this_tree.relpath(path))
+            if file_id is None:
+                continue
+            hash = self.this_tree.get_file_sha1(file_id)
+            if hash is None:
+                continue
+            modified_hashes[file_id] = hash
+        self.this_tree.set_merge_modified(modified_hashes)
+
     @staticmethod
     def parent(entry, file_id):
         """Determine the parent for a file_id (used as a key method)"""

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-01-27 05:28:12 +0000
+++ bzrlib/tests/test_commit.py	2006-03-06 22:11:12 +0000
@@ -366,3 +366,15 @@
             self.assertEqual(['called', 'called'], calls)
         finally:
             del bzrlib.ahook
+
+    def test_commit_object_doesnt_set_nick(self):
+        # using the Commit object directly does not set the branch nick.
+        wt = self.make_branch_and_tree('.')
+        c = Commit()
+        c.commit(working_tree=wt, message='empty tree', allow_pointless=True)
+        self.assertEquals(wt.branch.revno(), 1)
+        self.assertEqual({},
+                         wt.branch.repository.get_revision(
+                            wt.branch.last_revision()).properties)
+
+

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-02-17 14:55:36 +0000
+++ bzrlib/tests/test_inv.py	2006-03-06 15:27:23 +0000
@@ -490,3 +490,5 @@
         os.unlink('b1/a')
         wt.revert([])
         self.assertEqual(len(wt.inventory), 1)
+
+

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-02-22 14:48:36 +0000
+++ bzrlib/tests/test_transform.py	2006-02-24 19:00:42 +0000
@@ -77,12 +77,16 @@
         transform.create_file('contents', trans_id2)
         transform.set_executability(False, trans_id2)
         transform.version_file('my_pretties2', trans_id2)
-        transform.apply()
+        modified_paths = transform.apply().modified_paths
         self.assertEqual('contents', self.wt.get_file_byname('name').read())
         self.assertEqual(self.wt.path2id('name'), 'my_pretties')
         self.assertIs(self.wt.is_executable('my_pretties'), True)
         self.assertIs(self.wt.is_executable('my_pretties2'), False)
         self.assertEqual('directory', file_kind(self.wt.abspath('oz')))
+        self.assertEqual(len(modified_paths), 3)
+        tree_mod_paths = [self.wt.id2abspath(f) for f in 
+                          ('ozzie', 'my_pretties', 'my_pretties2')]
+        self.assertSubset(tree_mod_paths, modified_paths)
         # is it safe to finalize repeatedly?
         transform.finalize()
         transform.finalize()
@@ -554,6 +558,15 @@
         self.assertEqual(this.wt.get_file_byname('i.OTHER').read(),
                          'h\ni\nj\nk\n')
         self.assertEqual(os.path.exists(this.wt.abspath('i.BASE')), False)
+        modified = ['a', 'b', 'c', 'h', 'i']
+        merge_modified = this.wt.merge_modified()
+        self.assertSubset(merge_modified, modified)
+        self.assertEqual(len(merge_modified), len(modified))
+        file(this.wt.id2abspath('a'), 'wb').write('booga')
+        modified.pop(0)
+        merge_modified = this.wt.merge_modified()
+        self.assertSubset(merge_modified, modified)
+        self.assertEqual(len(merge_modified), len(modified))
 
     def test_file_merge(self):
         if not has_symlinks():

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-03-05 08:31:28 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-03-07 18:41:18 +0000
@@ -435,6 +435,32 @@
         self.assertEqual(1, old_tree.update())
         self.assertEqual('A', old_tree.last_revision())
 
+    def test_merge_revert(self):
+        from bzrlib.merge import merge_inner
+        this = self.make_branch_and_tree('b1')
+        open('b1/a', 'wb').write('a test\n')
+        this.add('a')
+        open('b1/b', 'wb').write('b test\n')
+        this.add('b')
+        this.commit(message='')
+        base = this.bzrdir.clone('b2').open_workingtree()
+        open('b2/a', 'wb').write('b test\n')
+        other = this.bzrdir.clone('b3').open_workingtree()
+        open('b3/a', 'wb').write('c test\n')
+        open('b3/c', 'wb').write('c test\n')
+        other.add('c')
+
+        open('b1/b', 'wb').write('q test\n')
+        open('b1/d', 'wb').write('d test\n')
+        merge_inner(this.branch, other, base, this_tree=this)
+        self.assertNotEqual(open('b1/a', 'rb').read(), 'a test\n')
+        this.revert([])
+        self.assertEqual(open('b1/a', 'rb').read(), 'a test\n')
+        self.assertIs(os.path.exists('b1/b~'), True)
+        self.assertIs(os.path.exists('b1/c'), False)
+        self.assertIs(os.path.exists('b1/a~'), False)
+        self.assertIs(os.path.exists('b1/d'), True)
+
     def test_update_updates_bound_branch_no_local_commits(self):
         # doing an update in a tree updates the branch its bound to too.
         master_tree = self.make_branch_and_tree('master')
@@ -473,3 +499,7 @@
         self.assertEqual([], tree.branch.revision_history())
         self.assertEqual(['bar'], tree.pending_merges())
 
+    def test_merge_modified(self):
+        tree = self.make_branch_and_tree('master')
+        tree._control_files.put('merge-hashes', StringIO('asdfasdf'))
+        self.assertRaises(errors.MergeModifiedFormatError, tree.merge_modified)

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-02-22 14:48:36 +0000
+++ bzrlib/transform.py	2006-03-07 18:41:18 +0000
@@ -36,6 +36,12 @@
     map[key] = value
 
 
+class _TransformResults(object):
+    def __init__(self, modified_paths):
+        object.__init__(self)
+        self.modified_paths = modified_paths
+
+
 class TreeTransform(object):
     """Represent a tree transformation.
     
@@ -660,10 +666,11 @@
         limbo_inv = {}
         inv = self._tree.inventory
         self._apply_removals(inv, limbo_inv)
-        self._apply_insertions(inv, limbo_inv)
+        modified_paths = self._apply_insertions(inv, limbo_inv)
         self._tree._write_inventory(inv)
         self.__done = True
         self.finalize()
+        return _TransformResults(modified_paths)
 
     def _limbo_name(self, trans_id):
         """Generate the limbo name of a file"""
@@ -711,6 +718,7 @@
         parent-to-child order.
         """
         new_paths = self.new_paths()
+        modified_paths = []
         for num, (path, trans_id) in enumerate(new_paths):
             self._pb.update('adding file', num+1, len(new_paths))
             try:
@@ -726,6 +734,7 @@
                     if e.errno != errno.ENOENT:
                         raise
                 if trans_id in self._new_contents:
+                    modified_paths.append(full_path)
                     del self._new_contents[trans_id]
 
             if trans_id in self._new_id:
@@ -744,6 +753,7 @@
             if trans_id in self._new_executability:
                 self._set_executability(path, inv, trans_id)
         self._pb.clear()
+        return modified_paths
 
     def _set_executability(self, path, inv, trans_id):
         """Set the executability of versioned files """
@@ -1002,6 +1012,7 @@
 
     tt = TreeTransform(working_tree, pb)
     try:
+        merge_modified = working_tree.merge_modified()
         trans_id = {}
         def trans_id_file_id(file_id):
             try:
@@ -1019,17 +1030,26 @@
                 e_trans_id = new_by_entry(tt, entry, parent_id, target_tree)
                 trans_id[file_id] = e_trans_id
             else:
+                backup_this = backups
+                if file_id in merge_modified:
+                    backup_this = False
+                    del merge_modified[file_id]
                 change_entry(tt, file_id, working_tree, target_tree, 
-                             trans_id_file_id, backups, trans_id)
+                             trans_id_file_id, backup_this, trans_id)
         wt_interesting = [i for i in working_tree.inventory if interesting(i)]
         for id_num, file_id in enumerate(wt_interesting):
             pb.update("New file check", id_num+1, len(sorted_interesting))
             if file_id not in target_tree:
-                tt.unversion_file(tt.trans_id_tree_file_id(file_id))
+                trans_id = tt.trans_id_tree_file_id(file_id)
+                tt.unversion_file(trans_id)
+                if file_id in merge_modified:
+                    tt.delete_contents(trans_id)
+                    del merge_modified[file_id]
         raw_conflicts = resolve_conflicts(tt, pb)
         for line in conflicts_strings(cook_conflicts(raw_conflicts, tt)):
             warning(line)
         tt.apply()
+        working_tree.set_merge_modified({})
     finally:
         tt.finalize()
         pb.clear()

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-03-06 07:11:09 +0000
+++ bzrlib/workingtree.py	2006-03-07 18:41:18 +0000
@@ -29,6 +29,7 @@
 WorkingTree.open(dir).
 """
 
+MERGE_MODIFIED_HEADER_1 = "BZR merge-modified list format 1"
 
 # FIXME: I don't know if writing out the cache from the destructor is really a
 # good idea, because destructors are considered poor taste in Python, and it's
@@ -60,7 +61,8 @@
                            WeaveRevisionNotPresent,
                            NotBranchError,
                            NoSuchFile,
-                           NotVersionedError)
+                           NotVersionedError,
+                           MergeModifiedFormatError)
 from bzrlib.inventory import InventoryEntry, Inventory
 from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.merge import merge_inner, transform_tree
@@ -84,6 +86,7 @@
                             )
 from bzrlib.progress import DummyProgress
 from bzrlib.revision import NULL_REVISION
+from bzrlib.rio import RioReader, RioWriter, Stanza
 from bzrlib.symbol_versioning import *
 from bzrlib.textui import show_status
 import bzrlib.tree
@@ -454,13 +457,18 @@
             tree.set_last_revision(revision_id)
 
     @needs_write_lock
-    def commit(self, *args, **kwargs):
+    def commit(self, message=None, revprops=None, *args, **kwargs):
+        # avoid circular imports
         from bzrlib.commit import Commit
+        if revprops is None:
+            revprops = {}
+        if not 'branch-nick' in revprops:
+            revprops['branch-nick'] = self.branch.nick
         # args for wt.commit start at message from the Commit.commit method,
         # but with branch a kwarg now, passing in args as is results in the
         #message being used for the branch
-        args = (DEPRECATED_PARAMETER, ) + args
-        Commit().commit(working_tree=self, *args, **kwargs)
+        args = (DEPRECATED_PARAMETER, message, ) + args
+        Commit().commit(working_tree=self, revprops=revprops, *args, **kwargs)
         self._set_inventory(self.read_working_inventory())
 
     def id2abspath(self, file_id):
@@ -600,6 +608,36 @@
     def set_pending_merges(self, rev_list):
         self._control_files.put_utf8('pending-merges', '\n'.join(rev_list))
 
+    @needs_write_lock
+    def set_merge_modified(self, modified_hashes):
+        my_file = StringIO()
+        my_file.write(MERGE_MODIFIED_HEADER_1 + '\n')
+        writer = RioWriter(my_file)
+        for file_id, hash in modified_hashes.iteritems():
+            s = Stanza(file_id=file_id, hash=hash)
+            writer.write_stanza(s)
+        my_file.seek(0)
+        self._control_files.put('merge-hashes', my_file)
+
+    @needs_read_lock
+    def merge_modified(self):
+        try:
+            hashfile = self._control_files.get('merge-hashes')
+        except NoSuchFile:
+            return {}
+        merge_hashes = {}
+        try:
+            if hashfile.next() != MERGE_MODIFIED_HEADER_1 + '\n':
+                raise MergeModifiedFormatError()
+        except StopIteration:
+            raise MergeModifiedFormatError()
+        for s in RioReader(hashfile):
+            file_id = s.get("file_id")
+            hash = s.get("hash")
+            if hash == self.get_file_sha1(file_id):
+                merge_hashes[file_id] = hash
+        return merge_hashes
+
     def get_symlink_target(self, file_id):
         return os.readlink(self.id2abspath(file_id))
 

